#pragma once
#include <string>

#include <glm/glm.hpp>

#include <imgui.h>
#include "../../libs/guizmo/ImGuizmo.h"

#include "../../YoggieEngine/src/YoggieEngine.h"
#include "../../src/Scene/Entity.h"

#include "EditorWindow.h"
#include "EditorConsole.h"
#include "../Project/Project.h"
#include "../AssetManagement/AssetManager.h"

typedef void ( *voidFunction ) (void);
using namespace YoggieEngine;

inline void ComponentView(const std::string& componentName, voidFunction func)
{
    ImGuiWindowFlags_ window_flags = ImGuiWindowFlags_None;
    ImGui::PushStyleVar(ImGuiStyleVar_ChildRounding, 5.0f);
    ImGui::BeginChild(componentName.c_str());

    func();

    ImGui::EndChild();
    ImGui::PopStyleVar();
}

class Inspector : EditorWindow {
public:
	Inspector() : EditorWindow("Inspector") {}

	void AddComponentDropDown(Entity& selected ) 
    {
		static char* names[] = { "Script Component", "Camera Component" };
        if (ImGui::Button("Add Component"))
            ImGui::OpenPopup("Component picker");

        ImGui::SameLine();
        if (ImGui::BeginPopup("Component picker")) {

            for (int i = 0; i < IM_ARRAYSIZE(names); i++)
                if (ImGui::MenuItem(names[i])) {
                    std::cout << "Add a " << names[i] << " to "
                        << selected.GetComponent<IdentifierComponent>().name << std::endl;
                }

            ImGui::EndPopup();
        }
        ImGui::NewLine();
	}
	
    void ShowComponents(Entity& selected) 
	{
        auto component = selected.GetComponent<IdentifierComponent>();
        char* buf = new char(component.name.size());
        strcpy(buf, component.name.c_str());
        ImGui::InputText("Name:", buf, sizeof(buf), ImGuiInputTextFlags_ReadOnly);


        if (selected.HasComponent<TransformComponent>()) {
            auto& transform = selected.GetComponent<TransformComponent>();
            if (ImGui::CollapsingHeader("Transform", ImGuiTreeNodeFlags_DefaultOpen)) {
                ImGui::DragFloat3("Position", glm::value_ptr(transform.Position), 0.01f);
                ImGui::DragFloat3("Rotation", glm::value_ptr(transform.Rotation), 0.01f);
                ImGui::DragFloat3("Scale", glm::value_ptr(transform.Scale), 0.01f, 0.0f);
            }

        }

        if (selected.HasComponent<Render3DComponent>()) {
            auto& render3d = selected.GetComponent<Render3DComponent>();
            if (ImGui::CollapsingHeader("Render3D", ImGuiTreeNodeFlags_DefaultOpen)) {
                ImGui::ColorEdit3("Colour", glm::value_ptr(render3d.color));
            }
        }

        if (selected.HasComponent<LightComponent>()) {
            auto& light = selected.GetComponent<LightComponent>();
            if (ImGui::CollapsingHeader("Light", ImGuiTreeNodeFlags_DefaultOpen)) {
                ImGui::DragFloat("Strength", &light.Strength, 0.001f);
                ImGui::ColorEdit3("Colour", glm::value_ptr(light.Color));
            }

        }

        if (selected.HasComponent <CameraComponent>()) {
            auto& camera = selected.GetComponent<CameraComponent>();
            static float Zoom = 90;
            static glm::vec3 Position, Rotation = glm::vec3(0.0f);
            ComponentView("Camera", [] {
                ImGui::SliderFloat("Zoom", &Zoom, 10, 190);
            ImGui::InputFloat3("Position:", &Position[0]);
            ImGui::InputFloat3("Rotation:", &Rotation[0]);
                });
        }

        if (selected.HasComponent<ScriptComponent>()) {
            ComponentView("Scripting", [] {
                ImGui::LabelText("##--", "Hello scripting");
                });
        }
	}

};

class SceneExplorer : EditorWindow {
public:
    SceneExplorer(entt::entity& selected , Scene& scene) : EditorWindow("SceneExplorer") {
        scene.getReg().each([&](entt::entity enttNumber) {
            Entity entity = Entity(enttNumber, &scene);
            auto id = entity.GetComponent<IdentifierComponent>();

            if (ImGui::Selectable(id.name.c_str(), enttNumber == selected)) {
                selected = enttNumber;
            }

        });

    }
};

class Viewport : EditorWindow {
public:
    Viewport (Framebuffer& fb) : EditorWindow("SceneView") {
      ImGui::Image(
            (void*)(intptr_t)fb.GetColourAttachment(),
            ImVec2{ (float)ImGui::GetWindowWidth(),(float)ImGui::GetWindowHeight() }
        );
        
        //ImGuizmo::SetDrawlist();
        //ImGuizmo::SetRect(ImGui::GetWindowPos().x, ImGui::GetWindowPos().y, ImGui::GetWindowWidth(), ImGui::GetWindowHeight());
        //ImGuizmo::Enable(true);
        //ImGuizmo::Manipulate(glm::value_ptr(view), glm::value_ptr(projection), ImGuizmo::TRANSLATE, ImGuizmo::WORLD, glm::value_ptr(trans));

    }

};

class Settings : EditorWindow {
public:
    Settings() : EditorWindow("Settings") {
        ImGui::LabelText("##title-settings", "Fine grain control over your engine... ");
    }
};

class ProjectInfo : EditorWindow {
public:
    ProjectInfo(Project& project) : EditorWindow("Project Info") {
        ImGui::Text("Project: %s", project.GetName().c_str());
        ImGui::Text("Directory: %s", project.GetProjectDirectory().u8string().c_str());
    }
};

class Console : EditorWindow {
public:

Console() : EditorWindow("Console") {
    s_console = std::make_unique<EditorConsole>();
}
void Show() {
    s_console.get()->Draw();
}

private: 
    std::unique_ptr<EditorConsole> s_console;
};

class AssetFinder : EditorWindow {
public:
    AssetFinder() : EditorWindow("Assets") {


        ImGui::DragInt("IconSize", &iconSize, 1, 30, 90);


        if (ImGui::BeginTable("##resources", 3))
        {
            ImGui::PushStyleColor(ImGuiCol_Button, ImVec4(0.f, 0.f, 0.f, 0.f));
            ImGui::PushStyleColor(ImGuiCol_ButtonActive, ImVec4(0.f, 0.f, 0.f, 0.f));
            ImGui::PushStyleColor(ImGuiCol_ButtonHovered, ImVec4(1.f, 1.f, 1.f, 0.2f));


            int row = 0;
            int column = 0;
            for (auto& asset : AssetManager::assets) {
                if (column % 3 == 0) {
                    ImGui::TableNextRow();
                    column = 0;
                    row++;
                }

                ImGui::TableSetColumnIndex(column);

                if (asset.isFolder) {
                    ImGui::ImageButton(
                        (ImTextureID)(Texture("rsc/folderIcon.png")).GetID(),
                        ImVec2{ (float)iconSize,(float)iconSize });
                    ImGui::Text(asset.GetName(), row);

                }
                else {
                    ImGui::ImageButton(
                        (ImTextureID)(Texture("rsc/assetIcon.png")).GetID(),
                        ImVec2{ (float)iconSize, (float)iconSize });
                    ImGui::Text(asset.GetName(), row);

                }


                column++;
            }


            ImGui::PopStyleColor(3);
            ImGui::EndTable();

        }

    }
private:
    static Texture folderIcon;
    static Texture AssetIcon;
    int iconSize = 60;
};